@@ -4,6 +4,7 @@ import copy
 import importlib
 import inspect
 import logging
+import warnings
 
 import pandas as pd
 from copulas.multivariate.gaussian import GaussianMultivariate
@@ -94,6 +95,12 @@ class Constraint(metaclass=ConstraintMeta):
     ``reverse_transform`` methods will be replaced respectively by a simple
     identity function.
 
+    Attributes:
+        constraint_columns (tuple[str]):
+            The names of the columns used by this constraint.
+        rebuild_columns (typle[str]):
+            The names of the columns that this constraint will rebuild during
+            ``reverse_transform``.
     Args:
         handling_strategy (str):
             How this Constraint should be handled, which can be ``transform``,
@@ -105,17 +112,19 @@ class Constraint(metaclass=ConstraintMeta):
     """
 
     constraint_columns = ()
+    rebuild_columns = ()
     _hyper_transformer = None
     _columns_model = None
 
     def _identity(self, table_data):
         return table_data
 
-    def __init__(self, handling_strategy, fit_columns_model=True):
+    def __init__(self, handling_strategy, fit_columns_model=False):
         self.fit_columns_model = fit_columns_model
         if handling_strategy == 'transform':
             self.filter_valid = self._identity
         elif handling_strategy == 'reject_sampling':
+            self.rebuild_columns = ()
             self.transform = self._identity
             self.reverse_transform = self._identity
         elif handling_strategy != 'all':
@@ -136,6 +145,8 @@ class Constraint(metaclass=ConstraintMeta):
             table_data (pandas.DataFrame):
                 Table data.
         """
+        self._fit(table_data)
+
         if self.fit_columns_model and len(self.constraint_columns) > 1:
             data_to_model = table_data[list(self.constraint_columns)]
             self._hyper_transformer = HyperTransformer(dtype_transformers={
@@ -145,28 +156,65 @@ class Constraint(metaclass=ConstraintMeta):
             self._columns_model = GaussianMultivariate()
             self._columns_model.fit(transformed_data)
 
-        return self._fit(table_data)
-
     def _transform(self, table_data):
         return table_data
 
+    def _reject_sample(self, num_rows, conditions):
+        sampled = self._columns_model.sample(
+            num_rows=num_rows,
+            conditions=conditions
+        )
+        sampled = self._hyper_transformer.reverse_transform(sampled)
+        valid_rows = sampled[self.is_valid(sampled)]
+        counter = 0
+        total_sampled = num_rows
+
+        while len(valid_rows) < num_rows:
+            num_valid = len(valid_rows)
+            if counter >= 100:
+                if len(valid_rows) == 0:
+                    error = 'Could not get enough valid rows within 100 trials.'
+                    raise ValueError(error)
+                else:
+                    multiplier = num_rows // num_valid
+                    num_rows_missing = num_rows % num_valid
+                    remainder_rows = valid_rows.iloc[0:num_rows_missing, :]
+                    valid_rows = pd.concat([valid_rows] * multiplier + [remainder_rows],
+                                           ignore_index=True)
+                    break
+
+            remaining = num_rows - num_valid
+            valid_probability = (num_valid + 1) / (total_sampled + 1)
+            max_rows = num_rows * 10
+            num_to_sample = min(int(remaining / valid_probability), max_rows)
+            total_sampled += num_to_sample
+            new_sampled = self._columns_model.sample(
+                num_rows=num_to_sample,
+                conditions=conditions
+            )
+            new_sampled = self._hyper_transformer.reverse_transform(new_sampled)
+            new_valid_rows = new_sampled[self.is_valid(new_sampled)]
+            valid_rows = pd.concat([valid_rows, new_valid_rows], ignore_index=True)
+            counter += 1
+
+        return valid_rows.iloc[0:num_rows, :]
+
     def _sample_constraint_columns(self, table_data):
         condition_columns = [c for c in self.constraint_columns if c in table_data.columns]
         grouped_conditions = table_data[condition_columns].groupby(condition_columns)
-        sampled_rows = list()
+        all_sampled_rows = list()
         for group, df in grouped_conditions:
             if not isinstance(group, tuple):
                 group = [group]
 
-            transformed_condition = self._hyper_transformer.transform(df).loc[0].to_dict()
-            sampled_row = self._columns_model.sample(
+            transformed_condition = self._hyper_transformer.transform(df).iloc[0].to_dict()
+            sampled_rows = self._reject_sample(
                 num_rows=df.shape[0],
                 conditions=transformed_condition
             )
-            sampled_rows.append(sampled_row)
+            all_sampled_rows.append(sampled_rows)
 
-        sampled_data = pd.concat(sampled_rows)
-        sampled_data = self._hyper_transformer.reverse_transform(sampled_data)
+        sampled_data = pd.concat(all_sampled_rows, ignore_index=True)
         return sampled_data
 
     def _validate_constraint_columns(self, table_data):
@@ -187,6 +235,15 @@ class Constraint(metaclass=ConstraintMeta):
         """
         missing_columns = [col for col in self.constraint_columns if col not in table_data.columns]
         if missing_columns:
+            if not self._columns_model:
+                warning_message = (
+                    'When `fit_columns_model` is False and we are conditioning on a subset '
+                    'of the constraint columns, conditional sampling uses reject sampling '
+                    'which can be slow. Changing `fit_columns_model` to True can improve '
+                    'the performance.'
+                )
+                warnings.warn(warning_message, UserWarning)
+
             all_columns_missing = len(missing_columns) == len(self.constraint_columns)
             if self._columns_model is None or all_columns_missing:
                 raise MissingConstraintColumnError()
